#!/usr/bin/env python
# -*- coding: utf-8 -*-
#

## @package Highlighter.py
#  La clase encargada de resaltar el
#  texto...
#
#  Para Toccata, IDE para Lilypond,
#  Futuro IDE para edicin completa de
#  Partituras, casi como WYSIWYG (espero)
# 
#  Extrada de los ejemplos de PyQt4.
#
#  Extendida por Javier Salazar Loyola <jsalazar@elo.utfsm.cl>


from PyQt4 import QtGui, QtCore

## Clase que se encarga de resaltar el texto.
#  Se basa en los mtodos para identificar texto de la
#  clase QtGui.QRegExp
#
class Highlighter(QtCore.QObject):

    ## El constructor, inicializa el diccionario en que se basa el resaltado
    #  @param parent QObject padre de la instancia
    #
    def __init__(self, parent=None):
        QtCore.QObject.__init__(self, parent)

        ## Diccionario con el formato del texto resaltado segn cada uno de los patrones
        #  que se destacarn
        self.mappings = {}
        
    ## Mtodo para conectar un determinado documento QtGui.QTextDocument con el resaltador,
    #  de forma de que la instancia sepa dnde debe resaltar.
    #
    #  @param doc QTextDocument al que se resaltar el texto
    def addToDocument(self, doc):
        self.connect(doc, QtCore.SIGNAL("contentsChange(int, int, int)"), self.highlight)
    

    ## Mtodo que actualiza los resaltados que se utilizarn
    #
    #  @param pattern Cadena con la informacin de los parmetros que se destacarn
    #  @param format Objeto de la clase QtGui.QTextCharFormat que indica cmo resaltar
    def addMapping(self, pattern, format):
        self.mappings[pattern] = format


    ## El resaltado es actualizado con cada cambio en el documento, este mtodo realida 
    #  ese cambio. Debido a que este mtodo se conecta con el cada cambio en el documento,
    #  aquella seal es la que entrega los parmetros necesarios para realizar la operacin.
    #
    #  @param position La posicin en que se realiz el cambio
    #  @param removed Cunto texto se removi en el cambio
    #  @param added Cunto texto se agreg en el cambio
    def highlight(self, position, removed, added):
        doc = self.sender() #el documento que cambi es el afectado
    
        block = doc.findBlock(position)#agarra la posicin
        if not block.isValid():
            return
    
        endBlock = QtGui.QTextBlock()
        if added > removed:#si se agreg ms de lo que se borr? o.o
            endBlock = doc.findBlock(position + added)#el siguiente bloque...
        else:
            endBlock = block
    
        while block.isValid() and not (endBlock < block):#mientras sea vlido el bloque y el siguiente sea ms largo
            self.highlightBlock(block)
            block = block.next()

    ## Mtodo que se realiza al cargar un archivo nuevo, con el objeto de resaltar el texto
    #  con cada carga de un nuevo archivo
    #
    #  @param doc El texto cargado (QtGui.QTextDocument)
    #  @return True cuando el texto ha sido resaltado completo
    def loadHighlight(self, doc):#para iniciar el recorrido...
        block = doc.begin()

        while block != doc.end():
            if block.isValid():
                self.highlightBlock(block)
                block = block.next()

        return True
        
    
    ## Este mtodo es el que realiza el resaltado en s.
    #
    #  @param block El bloque de texto que se resaltar.
    def highlightBlock(self, block):
        layout = block.layout()
        text = block.text()#.toLower()#provisional...
    
        overrides = [] #lista
    
        for pattern in self.mappings:
            expression = QtCore.QRegExp(pattern)
            i = text.indexOf(expression)
            while i >= 0:
                range = QtGui.QTextLayout.FormatRange() #clase de QTextLayout (no documentada .U)
                range.start = i
                range.length = expression.matchedLength()
                range.format = self.mappings[pattern] #uso del dicionario
                overrides.append(range) #agrega cosas a la lista
    
                i = text.indexOf(expression, i + expression.matchedLength()) #siguiente...
    
        layout.setAdditionalFormats(overrides)#agrega formatos...
        block.document().markContentsDirty(block.position(), block.length())
        #llama al documento y marca el bloque como "sucio" => debe ser repintado

