#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
########################################
## @package Subprocesos.py
#
# Mdulo que define una clase para trabajar
# con subprocesos segn la definicin de
# QtCore.QProcess y que define una clase
# para visualizar la salida de la compilacin
# con Lilypond, heredera de la clase QtGui.QTextEdit
#
#
#  Para Toccata, IDE para Lilypond,
#  Futuro IDE para edicin completa de
#  Partituras, casi como WYSIWYG (espero)
#
# @author Javier Salazar Loyola <jsalazar@elo.utfsm.cl>
#
########################################

from PyQt4 import QtGui, QtCore

##Clase que maneja los subprocesos del programa, siendo stos
# el compilador (Lilypond) y los visores de PDF y PS.
class ProcessHandler(QtCore.QObject):

    ##Crea un nuevo objeto con los programas requeridos.
    # @param parent Objeto padre de la instancia, de la clase MainWindow
    # @param nombreCompilador el comando a correr para compilar
    # @param nombreVisorPDF el comando a correr para visualizar el PDF
    # @param nombreVisorPS el comando a correr para mostrar el PS generado
    def __init__(self, parent, nombreCompilador = "lilypond",
                 nombreVisorPDF = "xpdf", nombreVisorPS = "evince"):
        QtCore.QObject.__init__(self,parent)

        ##El objeto padre. Debe ser una ventana principal de la clase MainWindow,
        # de lo contrario, se generarn problemas por llamados a mtodos que
        # otras clases no tienen
        # @see MainWindow.py
        self.parent = parent

        ##El proceso que compila
        self.compilador = QtCore.QProcess(self)
        self.compilador.setReadChannelMode(QtCore.QProcess.MergedChannels)
        
        #self.visorPDF = QtCore.QProcess(self)
        #self.visorPS = QtCore.QProcess(self)

        ##Diccionario con los nombres de los programas a utilizar.
        self.programas = {'compilador':nombreCompilador,
                          'visorPDF':nombreVisorPDF,
                          'visorPS':nombreVisorPS}

        #conecciones de seales...
        self.connect(self.compilador, QtCore.SIGNAL("readyReadStandardOutput()"),
                     self.parent.LogView.actualizar)
        
    ##Mtodo que realiza la compilacin. Pregunta primero si se debe guardar el archivo, para
    # evitar problemas de archivo inexistente
    # @return False si no se puede compilar por problemas de guardado de archivo
    def compilar(self):
        if self.parent.archivoActual.isEmpty():
            if not self.parent.eventoGuardar():
                QtGui.QMessageBox.information(self.parent,'Advertencia',
                                          "No se pudo guardar el archivo, no se podr compilar",
                                          QtGui.QMessageBox.Ok)
                return False
            pass

        archivo = QtCore.QStringList(self.parent.archivoActual)
        self.compilador.setWorkingDirectory(self.parent.directorio.absolutePath())
        self.compilador.start(self.programas['compilador'],archivo)

        #if not self.compilador.waitForStarted():
        #    print self.compilador.state()

        #if not self.compilador.waitForFinished():
        #    return False
            #print self.compilador.state()
            #print "wtf!"

    ##Mtodo que crea un proceso hijo que muestra el archivo PDF generado
    def verPDF(self):
        fileInfo = QtCore.QFileInfo(QtCore.QFile(self.parent.archivoActual))
        QtCore.QProcess.startDetached(self.programas['visorPDF'],
                                      QtCore.QStringList(fileInfo.absolutePath() + '/' + 
                                                         fileInfo.baseName() + ".pdf")
                                      )
        return

    ##Mtodo que crea un proceso hijo que muestra el archivo PS generado
    def verPS(self):
        fileInfo = QtCore.QFileInfo(QtCore.QFile(self.parent.archivoActual))
        QtCore.QProcess.startDetached(self.programas['visorPS'],
                                      QtCore.QStringList(fileInfo.absolutePath() + '/' +
                                                         fileInfo.baseName() + '.ps')
                                      )
        return
        
        
##Clase que extiende QtGui.QTextEdit. Esta clase permite tener el registro de salida de la
# compilacin con Lilypond.
class LogViewer(QtGui.QTextEdit):

    ##Constructor
    # @param parent Objeto padre de la instancia de este objeto
    # @param parentOfParent Objeto padre del padre de la instancia, que finalmente es el padre
    # de este objeto
    def __init__(self, parent = None, parentOfParent = None):
        QtGui.QTextEdit.__init__(self,parent)
        #Padre del objeto para efectos padres: una instancia de MainWindow
        self.parent = parentOfParent

        self.setReadOnly(True)

    ##Mtodo que permite guardar el registro de salida como archivo de texto (.txt)
    # return False si no se guarda el registro
    def salvarLog(self):
        fileName = QtGui.QFileDialog.getSaveFileName(self,'Guardar registro como...',
                                                     self.parent.directorio.absolutePath(),
                                                     "Texto plano (*.txt)")

        if fileName.isEmpty():
            return False

        if len(fileName) < 4 or fileName[-4:] != '.txt':
            fileName = fileName + '.txt'

        file = QtCore.QFile(fileName)
        if not file.open(QtCore.QFile.WriteOnly | QtCore.QFile.Text):#revisa que pueda guardarse
            QtGui.QMessageBox.critical(self, 'Guardar Registro',
                                       'No se puede guardar el archivo en la ubicacin especificada')
            return False
        
        salida = QtCore.QTextStream(file)
        QtGui.QApplication.setOverrideCursor(QtCore.Qt.WaitCursor) #cursor de espera
        salida << self.toPlainText()
        QtGui.QApplication.restoreOverrideCursor()
        
    ##Mtodo que actualiza el estado del registro, agregando el resultado de la compilacin.
    # Debe mejorarse para futuras versiones
    def actualizar(self):
        self.insertPlainText(QtCore.QString(
            self.parent.procesosHijos.compilador.readAll()))

    #def comenzar(self):
    #    self.setPlainText("Compilacin comenzada...\n")
        
